/*------------------------------------------------------------------------------
 * Example main module
 * Copyright (c) 2020 Arm Limited (or its affiliates). All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    main.c
 * Purpose: Main module
 *----------------------------------------------------------------------------*/

#include "cmsis_os2.h"                  // ::CMSIS:RTOS2

#include "peripherals.h"                // Keil::Board Support:SDK Project Template:Project_Template
#include "pin_mux.h"                    // Keil::Board Support:SDK Project Template:Project_Template
#include "board.h"                      // Keil::Board Support:SDK Project Template:Project_Template
#include "fsl_iomuxc.h"                 // NXP::Device:SDK Drivers:iomuxc


extern       void           app_main (void *arg);
extern const osThreadAttr_t app_main_attr;

/* Approximate loop delay in us */
void DelayLoop (uint32_t us) {
  volatile uint32_t cyc_us, loop;
  
  cyc_us = (SystemCoreClock + 1000000U-1U) / 1000000U;
  loop = cyc_us * us;
  while (loop) {
    loop--;
  }
}

/*------------------------------------------------------------------------------
 * main function
 *----------------------------------------------------------------------------*/
int main(void) {
  gpio_pin_config_t gpio_config = {kGPIO_DigitalOutput, 0U, kGPIO_NoIntmode};

  // System initialization
  BOARD_InitBootPeripherals();
  BOARD_InitBootPins();
  BOARD_InitBootClocks();
  BOARD_InitDebugConsole();

  IOMUXC_EnableMode(IOMUXC_GPR, kIOMUXC_GPR_ENET1TxClkOutputDir, true);

  // Update System Core Clock info
  SystemCoreClockUpdate();

  // Init GPIO1, pins 9 and 10 as digital outputs
  GPIO_PinInit(GPIO1,  9U, &gpio_config);
  GPIO_PinInit(GPIO1, 10U, &gpio_config);

  // Pull up the ENET_INT before RESET
  GPIO_PinWrite(GPIO1, 10U, 1U);

  // Reset PHY
  GPIO_PinWrite(GPIO1,  9U, 0U);
  DelayLoop(500U);
  GPIO_PinWrite(GPIO1,  9U, 1U);
  DelayLoop(500U);

  osKernelInitialize ();                        // Initialize CMSIS-RTOS2
  osThreadNew (app_main, NULL, &app_main_attr); // Create application main thread
  osKernelStart ();                             // Start thread execution

  for (;;) {}
}
